<?php

namespace Teknisa\CADCTB\Services;

use Zeedhi\Framework\DTO\Response;
use Teknisa\CADCTB\Factories\Ger09400ParametroContabilFactory;
use Teknisa\Libs\Util\DateUtil;
use Zeedhi\Framework\DataSource\DataSet;
use Doctrine\ORM\EntityManager;
use Teknisa\Libs\Util\Environment;

class Ger09400ParametroContabilService{

    protected $ger09400ParametroContabilFactory;
    protected $environment;
    protected $nrorg;
    
    const EMPRESA_DESTINO_CADASTRADA = "EMPRESA_DESTINO_CADASTRADA";
    const EMPRESA_ORIGEM_NAO_CADASTRADA = "EMPRESA_ORIGEM_NAO_CADASTRADA";

    public function __construct(EntityManager $entityManager, Environment $environment, Ger09400ParametroContabilFactory $ger09400ParametroContabilFactory) {
        $this->ger09400ParametroContabilFactory = $ger09400ParametroContabilFactory;
        $this->environment = $environment;
        $this->entityManager = $entityManager;
        $this->nrorg = $environment->getNrOrgTrab();
    }
    
    public function cadastramentoAutomatico($row){
        $row = self::preparaRow($row);
        if ($row['OPERACAOEMP'] === 'I'){
            $contopectbl = $this->ger09400ParametroContabilFactory->insereContopectbl($row);
        }else{
            $contopectbl = $this->ger09400ParametroContabilFactory->getContopectbl($row);
            
            if ($row['IDALTERAPERIODO'] === 'S')
                $contopectbl = $this->ger09400ParametroContabilFactory->updateContopectbl($contopectbl, $row);

            if ($row['IDALTERALANCAMENTO'] === 'S')
                $contopectbl = $this->ger09400ParametroContabilFactory->updateContopectbl($contopectbl, $row);
        }
        $this->entityManager->persist($contopectbl);
    }
    
    public function validateSaldoPositivo($cdempresa, $dtinicial){
        $params = [];
        $params['CDEMPRESA'] = $cdempresa;
        $params['DTINICIAL'] = $dtinicial;
        return $this->ger09400ParametroContabilFactory->validateSaldoPositivo($params);
    }
    
    public function validateLacnamentoPendenteRecalculo($cdempresa){
        $params = [];
        $params['CDEMPRESA'] = $cdempresa;
        return $this->ger09400ParametroContabilFactory->validateLacnamentoPendenteRecalculo($params);
    }
    
    public function preparaRow($rowContopectbl){
        $rowContopectbl['DTFINACCTBL'] = self::formataData($rowContopectbl['DTFINACCTBL']);
        $rowContopectbl['DTFINACONT'] = self::formataData($rowContopectbl['DTFINACONT']);
        $rowContopectbl['DTFINALANC'] = self::formataData($rowContopectbl['DTFINALANC']);
        $rowContopectbl['DTFINARELA'] = self::formataData($rowContopectbl['DTFINARELA']);
        $rowContopectbl['DTFININTCONTDFE'] = self::formataData($rowContopectbl['DTFININTCONTDFE']);
        $rowContopectbl['DTFININTCONTDFS'] = self::formataData($rowContopectbl['DTFININTCONTDFS']);
        $rowContopectbl['DTFININTCONTGPA'] = self::formataData($rowContopectbl['DTFININTCONTGPA']);
        $rowContopectbl['DTFININTCONTFIS'] = self::formataData($rowContopectbl['DTFININTCONTFIS']);
        $rowContopectbl['DTFININTCONTVEXT'] = self::formataData($rowContopectbl['DTFININTCONTVEXT']);
        $rowContopectbl['DTINICCONT'] = self::formataData($rowContopectbl['DTINICCONT']);
        $rowContopectbl['DTINICCTBL'] = self::formataData($rowContopectbl['DTINICCTBL']);
        $rowContopectbl['DTINICLANC'] = self::formataData($rowContopectbl['DTINICLANC']);
        $rowContopectbl['DTINICRELA'] = self::formataData($rowContopectbl['DTINICRELA']);
        $rowContopectbl['DTINIINTCONTDFE'] = self::formataData($rowContopectbl['DTINIINTCONTDFE']);
        $rowContopectbl['DTINIINTCONTDFS'] = self::formataData($rowContopectbl['DTINIINTCONTDFS']);
        $rowContopectbl['DTINIINTCONTGPA'] = self::formataData($rowContopectbl['DTINIINTCONTGPA']);
        $rowContopectbl['DTINIINTCONTFIS'] = self::formataData($rowContopectbl['DTINIINTCONTFIS']);
        $rowContopectbl['DTINIINTCONTVEXT'] = self::formataData($rowContopectbl['DTINIINTCONTVEXT']);
        $rowContopectbl['NRORG'] = $this->nrorg;
        return $rowContopectbl;
    }
    
    public function saveParametroContabil($rowContopectbl){
    	$connection = $this->entityManager->getConnection();
    	$connection->beginTransaction();
    	try{
    		$rowContopectbl = self::preparaRow($rowContopectbl);
    		
    		$contopectbl = $this->ger09400ParametroContabilFactory->getContopectbl($rowContopectbl);
    		if($contopectbl)
    			 $contopectbl = $this->ger09400ParametroContabilFactory->updateContopectbl($contopectbl, $rowContopectbl);
    		else $contopectbl = $this->ger09400ParametroContabilFactory->insereContopectbl($rowContopectbl);
    
    		$this->entityManager->persist($contopectbl);
    		$this->entityManager->flush();
    		$connection->commit();
    	}catch(\Exception $e){
    		$connection->rollback();
    		throw $e;
    	}
    }


    public static function formataData($data){
        try {
            if($data == "Invalid date"){
                throw new \Exception ('Data inválida.');
            } else {
                if($data == "" ){
                    $data = null;
                }
                if($data != null){
                    if (strlen($data) == 19)
                        $data = DateUtil::truncateString($data);
                    $data = DateUtil::getDatadeString($data);
                    $data = DateUtil::truncateData($data);
                }
            }
            return $data;
        } catch(\Exception $e) {
            throw $e;
        }
    }
}